"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Observable = void 0;
var Subscriber_1 = require("./Subscriber");
var observable_1 = require("./symbol/observable");
var pipe_1 = require("./util/pipe");
var Observable = (function () {
    function Observable(subscribe) {
        if (subscribe) {
            this._subscribe = subscribe;
        }
    }
    Observable.prototype.subscribe = function (observerOrNext) {
        var subscriber = observerOrNext instanceof Subscriber_1.Subscriber ? observerOrNext : new Subscriber_1.Subscriber(observerOrNext);
        subscriber.add(this._trySubscribe(subscriber));
        return subscriber;
    };
    Observable.prototype._trySubscribe = function (sink) {
        try {
            return this._subscribe(sink);
        }
        catch (err) {
            sink.error(err);
        }
    };
    Observable.prototype.forEach = function (next) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var subscriber = new Subscriber_1.Subscriber({
                next: function (value) {
                    try {
                        next(value);
                    }
                    catch (err) {
                        reject(err);
                        subscriber.unsubscribe();
                    }
                },
                error: reject,
                complete: resolve,
            });
            _this.subscribe(subscriber);
        });
    };
    Observable.prototype._subscribe = function (_subscriber) {
        return;
    };
    Observable.prototype[observable_1.observable] = function () {
        return this;
    };
    Observable.prototype.pipe = function () {
        var operations = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operations[_i] = arguments[_i];
        }
        return (0, pipe_1.pipeFromArray)(operations)(this);
    };
    Observable.prototype[Symbol.asyncIterator] = function () {
        var _a;
        var _this = this;
        var subscription;
        var hasError = false;
        var error;
        var completed = false;
        var values = [];
        var deferreds = [];
        var handleError = function (err) {
            hasError = true;
            error = err;
            while (deferreds.length) {
                var _a = __read(deferreds.shift(), 2), _1 = _a[0], reject = _a[1];
                reject(err);
            }
        };
        var handleComplete = function () {
            completed = true;
            while (deferreds.length) {
                var _a = __read(deferreds.shift(), 1), resolve = _a[0];
                resolve({ value: undefined, done: true });
            }
        };
        return _a = {
                next: function () {
                    if (!subscription) {
                        subscription = _this.subscribe({
                            next: function (value) {
                                if (deferreds.length) {
                                    var _a = __read(deferreds.shift(), 1), resolve = _a[0];
                                    resolve({ value: value, done: false });
                                }
                                else {
                                    values.push(value);
                                }
                            },
                            error: handleError,
                            complete: handleComplete,
                        });
                    }
                    if (values.length) {
                        return Promise.resolve({ value: values.shift(), done: false });
                    }
                    if (completed) {
                        return Promise.resolve({ value: undefined, done: true });
                    }
                    if (hasError) {
                        return Promise.reject(error);
                    }
                    return new Promise(function (resolve, reject) {
                        deferreds.push([resolve, reject]);
                    });
                },
                throw: function (err) {
                    subscription === null || subscription === void 0 ? void 0 : subscription.unsubscribe();
                    handleError(err);
                    return Promise.reject(err);
                },
                return: function () {
                    subscription === null || subscription === void 0 ? void 0 : subscription.unsubscribe();
                    handleComplete();
                    return Promise.resolve({ value: undefined, done: true });
                }
            },
            _a[Symbol.asyncIterator] = function () {
                return this;
            },
            _a;
    };
    return Observable;
}());
exports.Observable = Observable;
//# sourceMappingURL=Observable.js.map