"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.single = void 0;
var Observable_1 = require("../Observable");
var EmptyError_1 = require("../util/EmptyError");
var SequenceError_1 = require("../util/SequenceError");
var NotFoundError_1 = require("../util/NotFoundError");
var OperatorSubscriber_1 = require("./OperatorSubscriber");
function single(predicate) {
    return function (source) {
        return new Observable_1.Observable(function (subscriber) {
            var hasValue = false;
            var singleValue;
            var seenValue = false;
            var index = 0;
            source.subscribe((0, OperatorSubscriber_1.createOperatorSubscriber)(subscriber, function (value) {
                seenValue = true;
                if (!predicate || predicate(value, index++, source)) {
                    hasValue && subscriber.error(new SequenceError_1.SequenceError('Too many matching values'));
                    hasValue = true;
                    singleValue = value;
                }
            }, function () {
                if (hasValue) {
                    subscriber.next(singleValue);
                    subscriber.complete();
                }
                else {
                    subscriber.error(seenValue ? new NotFoundError_1.NotFoundError('No matching values') : new EmptyError_1.EmptyError());
                }
            }));
        });
    };
}
exports.single = single;
//# sourceMappingURL=single.js.map