import { Observable } from '../Observable';
import { arrayOrObject } from '../util/argsArgArrayOrObject';
import { identity } from '../util/identity';
import { createObject } from '../util/createObject';
import { createOperatorSubscriber } from '../operators/OperatorSubscriber';
import { EMPTY } from './empty';
import { from } from './from';
export function combineLatest(sources, resultSelector) {
    const parts = arrayOrObject(sources);
    if (!parts) {
        throw new TypeError('sources must be an array or object');
    }
    const { args: observables, keys } = parts;
    if (observables.length === 0) {
        return EMPTY;
    }
    return new Observable(combineLatestInit(observables, keys
        ? (values) => createObject(keys, values)
        : resultSelector
            ? (values) => resultSelector(...values)
            : identity));
}
export function combineLatestInit(observables, valueTransform = identity) {
    return (subscriber) => {
        const { length } = observables;
        const values = new Array(length);
        let active = length;
        let remainingFirstValues = length;
        for (let i = 0; i < length; i++) {
            const source = from(observables[i]);
            let hasFirstValue = false;
            source.subscribe(createOperatorSubscriber(subscriber, (value) => {
                values[i] = value;
                if (!hasFirstValue) {
                    hasFirstValue = true;
                    remainingFirstValues--;
                }
                if (!remainingFirstValues) {
                    subscriber.next(valueTransform(Array.from(values)));
                }
            }, () => {
                if (!--active) {
                    subscriber.complete();
                }
            }));
        }
    };
}
//# sourceMappingURL=combineLatest.js.map