import { asyncScheduler } from '../scheduler/async';
import { Observable } from '../Observable';
import { createOperatorSubscriber } from './OperatorSubscriber';
export function debounceTime(dueTime, scheduler = asyncScheduler) {
    return (source) => new Observable((subscriber) => {
        let activeTask = null;
        let lastValue = null;
        let lastTime = null;
        let scheduling = false;
        const emit = () => {
            if (scheduling || activeTask) {
                if (activeTask) {
                    activeTask.unsubscribe();
                    activeTask = null;
                }
                const value = lastValue;
                lastValue = null;
                subscriber.next(value);
            }
        };
        function emitWhenIdle() {
            const targetTime = lastTime + dueTime;
            const now = scheduler.now();
            if (now < targetTime) {
                activeTask = this.schedule(undefined, targetTime - now);
                subscriber.add(activeTask);
                return;
            }
            emit();
        }
        source.subscribe(createOperatorSubscriber(subscriber, (value) => {
            lastValue = value;
            lastTime = scheduler.now();
            if (!activeTask) {
                scheduling = true;
                activeTask = scheduler.schedule(emitWhenIdle, dueTime);
                scheduling = false;
                subscriber.add(activeTask);
            }
        }, () => {
            emit();
            subscriber.complete();
        }, undefined, () => {
            lastValue = activeTask = null;
        }));
    });
}
//# sourceMappingURL=debounceTime.js.map